==> openturns_0.12.2.orig.tar.gz: openturns-0.12.2/lib/src/Base/Common/Log.cxx <==
//                                               -*- C++ -*-
/**
 *  @file  Log.cxx
 *  @brief Log records all user information to a file or tty
 *
 *  (C) Copyright 2005-2007 EDF-EADS-Phimeca
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License.
 *
 *  This library is distributed in the hope that it will be useful
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  @author: $LastChangedBy: dutka $
 *  @date:   $LastChangedDate: 2008-10-29 18:54:45 +0100 (mer 29 oct 2008) $
 *  Id:      $Id: Log.cxx 990 2008-10-29 17:54:45Z dutka $
 */
#include <iostream>
#include <cstdlib>
#include <assert.h>
#include <errno.h>

#if HAVE_PTHREAD_H
#include <pthread.h>
#endif

#include "OSS.hxx"
#include "Log.hxx"

namespace OpenTURNS
{

  namespace Base
  {

    namespace Common
    {

      Log * Log::P_instance_ = 0;
      pthread_mutex_t instanceMutex = PTHREAD_MUTEX_INITIALIZER;


      const Log::Severity Log::NONE    =  0;
      const Log::Severity Log::ALL     = ~0;

      const Log::Severity Log::DBG     = 1 << 0;
      const Log::Severity Log::WRAPPER = 1 << 1;
      const Log::Severity Log::INFO    = 1 << 2;
      const Log::Severity Log::USER    = 1 << 3;
      const Log::Severity Log::WARN    = 1 << 4;
      const Log::Severity Log::ERROR   = 1 << 5;

      Log::Severity Log::Severity_ = Log::USER | Log::WARN | Log::ERROR;

      /* Constructor */
      Log::Log()
	: p_file_(0),
	  prefix_()
      {
	prefix_[DBG]     = "DBG - ";
	prefix_[WRAPPER] = "WRP - ";
	prefix_[INFO]    = "INF - ";
	prefix_[USER]    = "USR - ";
	prefix_[WARN]    = "WRN - ";
	prefix_[ERROR]   = "ERR - ";

	// Registration of destructor at exit
	std::atexit(Log::Release);
	push(Entry(INFO, "*** Log Beginning ***"));
      }


      /* Destructor */
      Log::~Log()
      {
	push(Entry(INFO, "*** Log End ***"));

	delete p_file_;
	p_file_ = 0;
      }



      Log & Log::GetInstance()
      {
	if (!P_instance_) P_instance_ = new Log;
	// std::cerr << "Log::GetInstance() P_instance_ = " << P_instance_ << std::endl;
	return *P_instance_;
      }



      void Log::Release()
      {
	delete P_instance_;
	P_instance_ = 0;
      }



      /* Log messages according to its relative severity */
      void Log::Debug(const String & msg)
      {
	pthread_mutex_lock( &instanceMutex );
	Log::GetInstance().push(Entry(DBG, msg));
	pthread_mutex_unlock( &instanceMutex );
      }


      
      /* Log messages according to its relative severity */
      void Log::Wrapper(const String & msg)
      {
	pthread_mutex_lock( &instanceMutex );
	Log::GetInstance().push(Entry(WRAPPER, msg));
	pthread_mutex_unlock( &instanceMutex );
      }


      
      /*  Log messages according to its relative severity */
      void Log::Info(const String & msg)
      {
	pthread_mutex_lock( &instanceMutex );
	Log::GetInstance().push(Entry(INFO, msg));
	pthread_mutex_unlock( &instanceMutex );
      }



      /*  Log messages according to its relative severity */
      void Log::User(const String & msg)
      {
	pthread_mutex_lock( &instanceMutex );
	Log::GetInstance().push(Entry(USER, msg));
	pthread_mutex_unlock( &instanceMutex );
      }



      /* Log messages according to its relative severity */
      void Log::Warn(const String & msg)
      {
	pthread_mutex_lock( &instanceMutex );
	Log::GetInstance().push(Entry(WARN, msg));
	pthread_mutex_unlock( &instanceMutex );
      }



      /* Log messages according to its relative severity */
      void Log::Error(const String & msg)
      {
	pthread_mutex_lock( &instanceMutex );
	Log::GetInstance().push(Entry(ERROR, msg));
	pthread_mutex_unlock( &instanceMutex );
      }


      /* Get/Set the severity flags for the messages logged to the file */
      void Log::Show(Severity flags)
      {
	Log::Severity_ = flags;
      }

      Log::Severity Log::Flags()
      {
	return Log::Severity_;
      }

     
      /* Append an entry at the end of the list */
      void Log::push(const Entry & entry)
      {
	std::ostream & os = p_file_ ? *p_file_ : std::clog;
	if (entry.sev_ & Log::Severity_) os << prefix_[entry.sev_] << entry.msg_ << std::endl;
      }


      /* Set the name of the log file */
      void Log::SetFile(const FileName & file)
      {
	Log::GetInstance().setFile(file);
      }


      /* Set the name of the log file */
      void Log::setFile(const FileName & file)
      {
	push(Entry(INFO, OSS() << "Diverting log to file: " << file));
	push(Entry(INFO, "*** Log End ***"));
	delete p_file_;
	p_file_ = new std::ofstream(file.c_str());

	push(Entry(INFO, "*** Log Beginning ***"));
      }




    } /* namespace Common */
  } /* namespace Base */
} /* namespace OpenTURNS */

==> openturns_0.13.1.orig.tar.gz: openturns-0.13.1/lib/src/Base/Common/Log.cxx <==
//                                               -*- C++ -*-
/**
 *  @file  Log.cxx
 *  @brief Log records all user information to a file or tty
 *
 *  (C) Copyright 2005-2007 EDF-EADS-Phimeca
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License.
 *
 *  This library is distributed in the hope that it will be useful
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  @author: $LastChangedBy: dutka $
 *  @author: $LastChangedBy: dutka $
 *  @date:   $LastChangedDate: 2009-05-28 18:00:31 +0200 (jeu. 28 mai 2009) $
 *  Id:      $Id: Log.cxx 1263 2009-05-28 16:00:31Z dutka $
 */
#include <iostream>
#include <cstdlib>
#include <cassert>
#include <errno.h>

#include "OTconfig.hxx"
#include "OSS.hxx"
#include "Log.hxx"
#include "OTconfig.hxx"

#if HAVE_PTHREAD_H

#include <pthread.h>
#ifdef WIN32
// MinGW pthread.h includes windows.h which define a HUGE number of macros..
#undef ERROR
#endif

#endif


namespace OpenTURNS
{

  namespace Base
  {

    namespace Common
    {

      Log * Log::P_instance_ = 0;
      pthread_mutex_t instanceMutex = PTHREAD_MUTEX_INITIALIZER;


      const Log::Severity Log::NONE    =  0;
      const Log::Severity Log::ALL     = ~0;

      const Log::Severity Log::DBG     = 1 << 0;
      const Log::Severity Log::WRAPPER = 1 << 1;
      const Log::Severity Log::INFO    = 1 << 2;
      const Log::Severity Log::USER    = 1 << 3;
      const Log::Severity Log::WARN    = 1 << 4;
      const Log::Severity Log::ERROR   = 1 << 5;

      Log::Severity Log::Severity_ = Log::USER | Log::WARN | Log::ERROR;

      /* Constructor */
      Log::Log()
	: p_file_(0),
	  prefix_(),
	  previousMessage_(),
	  count_(0)
      {
	prefix_[DBG]     = "DBG - ";
	prefix_[WRAPPER] = "WRP - ";
	prefix_[INFO]    = "INF - ";
	prefix_[USER]    = "USR - ";
	prefix_[WARN]    = "WRN - ";
	prefix_[ERROR]   = "ERROR - ";

	// Registration of destructor at exit
	std::atexit(Log::Release);
	push(Entry(INFO, "*** Log Beginning ***"));
      }


      /* Destructor */
      Log::~Log()
      {
	push(Entry(INFO, "*** Log End ***"));

	delete p_file_;
	p_file_ = 0;
      }



      Log & Log::GetInstance()
      {
	if (!P_instance_) P_instance_ = new Log;
	// std::cerr << "Log::GetInstance() P_instance_ = " << P_instance_ << std::endl;
	return *P_instance_;
      }



      void Log::Release()
      {
	delete P_instance_;
	P_instance_ = 0;
      }



      /* Log messages according to its relative severity */
      void Log::Debug(const String & msg)
      {
	pthread_mutex_lock( &instanceMutex );
	Log::GetInstance().push(Entry(DBG, msg));
	pthread_mutex_unlock( &instanceMutex );
      }


      
      /* Log messages according to its relative severity */
      void Log::Wrapper(const String & msg)
      {
	pthread_mutex_lock( &instanceMutex );
	Log::GetInstance().push(Entry(WRAPPER, msg));
	pthread_mutex_unlock( &instanceMutex );
      }


      
      /*  Log messages according to its relative severity */
      void Log::Info(const String & msg)
      {
	pthread_mutex_lock( &instanceMutex );
	Log::GetInstance().push(Entry(INFO, msg));
	pthread_mutex_unlock( &instanceMutex );
      }



      /*  Log messages according to its relative severity */
      void Log::User(const String & msg)
      {
	pthread_mutex_lock( &instanceMutex );
	Log::GetInstance().push(Entry(USER, msg));
	pthread_mutex_unlock( &instanceMutex );
      }



      /* Log messages according to its relative severity */
      void Log::Warn(const String & msg)
      {
	pthread_mutex_lock( &instanceMutex );
	Log::GetInstance().push(Entry(WARN, msg));
	pthread_mutex_unlock( &instanceMutex );
      }



      /* Log messages according to its relative severity */
      void Log::Error(const String & msg)
      {
	pthread_mutex_lock( &instanceMutex );
	Log::GetInstance().push(Entry(ERROR, msg));
	pthread_mutex_unlock( &instanceMutex );
      }


      /* Get/Set the severity flags for the messages logged to the file */
      void Log::Show(Severity flags)
      {
	Log::Severity_ = flags;
      }

      Log::Severity Log::Flags()
      {
	return Log::Severity_;
      }

      /* Flush pending messages */
      void Log::Flush()
      {
	Log::GetInstance().flush();
      }


      void Log::flush()
      {
	printRepeatedMessage( previousMessage_ );
	previousMessage_ = Entry();
	count_ = 0;
      }


      /* Append an entry at the end of the list */
      void Log::push(const Entry & entry)
      {
	std::ostream & os = p_file_ ? *p_file_ : std::clog;
	if (entry.sev_ & Log::Severity_) {
	  if (entry == previousMessage_) ++count_;
	  else {
	    printRepeatedMessage( previousMessage_ );
	    previousMessage_ = entry ;
	    count_ = 0;
	    os << prefix_[entry.sev_] << entry.msg_ << std::endl;
	  }
	}
      }

      void Log::printRepeatedMessage(const Entry & entry)
      {
	std::ostream & os = p_file_ ? *p_file_ : std::clog;
	if (count_ > 0)
	  os << prefix_[entry.sev_] << "(previous message repeated " << count_ << " time" << ((count_ == 1) ? "" : "s") << ")" << std::endl;
      }

      /* Set the name of the log file */
      void Log::SetFile(const FileName & file)
      {
	Log::GetInstance().setFile(file);
      }


      /* Set the name of the log file */
      void Log::setFile(const FileName & file)
      {
	push(Entry(INFO, OSS() << "Diverting log to file: " << file));
	push(Entry(INFO, "*** Log End ***"));
	delete p_file_;
	p_file_ = new std::ofstream(file.c_str());

	push(Entry(INFO, "*** Log Beginning ***"));
      }




    } /* namespace Common */
  } /* namespace Base */
} /* namespace OpenTURNS */
